{-# OPTIONS_GHC -Wall #-}
{-# OPTIONS_GHC -fno-warn-unused-imports #-}
{-# OPTIONS_GHC -fno-warn-orphans #-}
{-# Language ForeignFunctionInterface #-}
{-# Language FlexibleInstances #-}
{-# Language MultiParamTypeClasses #-}

module Casadi.Wrappers.Classes.OptionsFunctionality
       (
         OptionsFunctionality,
         OptionsFunctionalityClass(..),
         optionsFunctionality,
         optionsFunctionality_checkNode,
         optionsFunctionality_copyOptions,
         optionsFunctionality_copyOptions',
         optionsFunctionality_getOption,
         optionsFunctionality_getOptionAllowed,
         optionsFunctionality_getOptionAllowedIndex,
         optionsFunctionality_getOptionDefault,
         optionsFunctionality_getOptionDescription,
         optionsFunctionality_getOptionEnumValue,
         optionsFunctionality_getOptionNames,
         optionsFunctionality_getOptionType,
         optionsFunctionality_getOptionTypeName,
         optionsFunctionality_hasOption,
         optionsFunctionality_hasSetOption,
         optionsFunctionality_printOptions',
         optionsFunctionality_setOption,
         optionsFunctionality_setOptionByAllowedIndex,
         optionsFunctionality_setOptionByEnumValue,
       ) where


import Prelude hiding ( Functor )

import Data.Vector ( Vector )
import Foreign.C.Types
import Foreign.Ptr ( Ptr )
import Foreign.ForeignPtr ( newForeignPtr )
import System.IO.Unsafe ( unsafePerformIO ) -- for show instances

import Casadi.Wrappers.Classes.PrintableObject
import Casadi.Wrappers.CToolsInstances ( )
import Casadi.Wrappers.Data
import Casadi.Wrappers.Enums
import Casadi.MarshalTypes ( CppVec, StdString' ) -- StdOstream'
import Casadi.Marshal ( Marshal(..), withMarshal )
import Casadi.WrapReturn ( WrapReturn(..) )

instance Show OptionsFunctionality where
  show = unsafePerformIO . printableObject_getDescription
-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__setOption" c_CasADi__OptionsFunctionality__setOption
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> Ptr GenericType' -> IO ()
casADi__OptionsFunctionality__setOption
  :: OptionsFunctionality -> String -> GenericType -> IO ()
casADi__OptionsFunctionality__setOption x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__OptionsFunctionality__setOption x0' x1' x2' >>= wrapReturn

-- classy wrapper
{-|
>>  void CasADi::OptionsFunctionality::setOption(const std::string &str, const GenericType &val)
>------------------------------------------------------------------------
>
>set an option. For a list of options, check the class documentation of this
>class.
>
>The setOptions are only considered before the init function. If properties
>changes, the init function should be called again.
>
>>  void CasADi::OptionsFunctionality::setOption(const Dictionary &dict, bool skipUnknown=false)
>------------------------------------------------------------------------
>
>set a set of options. For a list of options, check the class documentation
>of this class.
>
>The setOptions are only considered before the init function. If properties
>changes, the init function should be called again.
-}
optionsFunctionality_setOption :: OptionsFunctionalityClass a => a -> String -> GenericType -> IO ()
optionsFunctionality_setOption x = casADi__OptionsFunctionality__setOption (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__getOption" c_CasADi__OptionsFunctionality__getOption
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> IO (Ptr GenericType')
casADi__OptionsFunctionality__getOption
  :: OptionsFunctionality -> String -> IO GenericType
casADi__OptionsFunctionality__getOption x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__getOption x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>get an option value
-}
optionsFunctionality_getOption :: OptionsFunctionalityClass a => a -> String -> IO GenericType
optionsFunctionality_getOption x = casADi__OptionsFunctionality__getOption (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__hasOption" c_CasADi__OptionsFunctionality__hasOption
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> IO CInt
casADi__OptionsFunctionality__hasOption
  :: OptionsFunctionality -> String -> IO Bool
casADi__OptionsFunctionality__hasOption x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__hasOption x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>check if there is an option str
-}
optionsFunctionality_hasOption :: OptionsFunctionalityClass a => a -> String -> IO Bool
optionsFunctionality_hasOption x = casADi__OptionsFunctionality__hasOption (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__hasSetOption" c_CasADi__OptionsFunctionality__hasSetOption
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> IO CInt
casADi__OptionsFunctionality__hasSetOption
  :: OptionsFunctionality -> String -> IO Bool
casADi__OptionsFunctionality__hasSetOption x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__hasSetOption x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>check if the user has there is an option str
-}
optionsFunctionality_hasSetOption :: OptionsFunctionalityClass a => a -> String -> IO Bool
optionsFunctionality_hasSetOption x = casADi__OptionsFunctionality__hasSetOption (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__printOptions_TIC" c_CasADi__OptionsFunctionality__printOptions_TIC
  :: Ptr OptionsFunctionality' -> IO ()
casADi__OptionsFunctionality__printOptions'
  :: OptionsFunctionality -> IO ()
casADi__OptionsFunctionality__printOptions' x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__OptionsFunctionality__printOptions_TIC x0' >>= wrapReturn

-- classy wrapper
optionsFunctionality_printOptions' :: OptionsFunctionalityClass a => a -> IO ()
optionsFunctionality_printOptions' x = casADi__OptionsFunctionality__printOptions' (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__copyOptions" c_CasADi__OptionsFunctionality__copyOptions
  :: Ptr OptionsFunctionality' -> Ptr OptionsFunctionality' -> CInt -> IO ()
casADi__OptionsFunctionality__copyOptions
  :: OptionsFunctionality -> OptionsFunctionality -> Bool -> IO ()
casADi__OptionsFunctionality__copyOptions x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__OptionsFunctionality__copyOptions x0' x1' x2' >>= wrapReturn

-- classy wrapper
{-|
>Copy all options from another object.
-}
optionsFunctionality_copyOptions :: OptionsFunctionalityClass a => a -> OptionsFunctionality -> Bool -> IO ()
optionsFunctionality_copyOptions x = casADi__OptionsFunctionality__copyOptions (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__copyOptions_TIC" c_CasADi__OptionsFunctionality__copyOptions_TIC
  :: Ptr OptionsFunctionality' -> Ptr OptionsFunctionality' -> IO ()
casADi__OptionsFunctionality__copyOptions'
  :: OptionsFunctionality -> OptionsFunctionality -> IO ()
casADi__OptionsFunctionality__copyOptions' x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__copyOptions_TIC x0' x1' >>= wrapReturn

-- classy wrapper
optionsFunctionality_copyOptions' :: OptionsFunctionalityClass a => a -> OptionsFunctionality -> IO ()
optionsFunctionality_copyOptions' x = casADi__OptionsFunctionality__copyOptions' (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__checkNode" c_CasADi__OptionsFunctionality__checkNode
  :: Ptr OptionsFunctionality' -> IO CInt
casADi__OptionsFunctionality__checkNode
  :: OptionsFunctionality -> IO Bool
casADi__OptionsFunctionality__checkNode x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__OptionsFunctionality__checkNode x0' >>= wrapReturn

-- classy wrapper
{-|
>Assert that the node is pointing to the right type of object.
-}
optionsFunctionality_checkNode :: OptionsFunctionalityClass a => a -> IO Bool
optionsFunctionality_checkNode x = casADi__OptionsFunctionality__checkNode (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__getOptionNames" c_CasADi__OptionsFunctionality__getOptionNames
  :: Ptr OptionsFunctionality' -> IO (Ptr (CppVec (Ptr StdString')))
casADi__OptionsFunctionality__getOptionNames
  :: OptionsFunctionality -> IO (Vector String)
casADi__OptionsFunctionality__getOptionNames x0 =
  withMarshal x0 $ \x0' ->
  c_CasADi__OptionsFunctionality__getOptionNames x0' >>= wrapReturn

-- classy wrapper
{-|
>Get a list of all option names.
-}
optionsFunctionality_getOptionNames :: OptionsFunctionalityClass a => a -> IO (Vector String)
optionsFunctionality_getOptionNames x = casADi__OptionsFunctionality__getOptionNames (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__getOptionDescription" c_CasADi__OptionsFunctionality__getOptionDescription
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> IO (Ptr StdString')
casADi__OptionsFunctionality__getOptionDescription
  :: OptionsFunctionality -> String -> IO String
casADi__OptionsFunctionality__getOptionDescription x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__getOptionDescription x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Get the description of a certain option.
-}
optionsFunctionality_getOptionDescription :: OptionsFunctionalityClass a => a -> String -> IO String
optionsFunctionality_getOptionDescription x = casADi__OptionsFunctionality__getOptionDescription (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__getOptionType" c_CasADi__OptionsFunctionality__getOptionType
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> IO CInt
casADi__OptionsFunctionality__getOptionType
  :: OptionsFunctionality -> String -> IO Opt_type
casADi__OptionsFunctionality__getOptionType x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__getOptionType x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Get the type of a certain option.
-}
optionsFunctionality_getOptionType :: OptionsFunctionalityClass a => a -> String -> IO Opt_type
optionsFunctionality_getOptionType x = casADi__OptionsFunctionality__getOptionType (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__getOptionTypeName" c_CasADi__OptionsFunctionality__getOptionTypeName
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> IO (Ptr StdString')
casADi__OptionsFunctionality__getOptionTypeName
  :: OptionsFunctionality -> String -> IO String
casADi__OptionsFunctionality__getOptionTypeName x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__getOptionTypeName x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Get the type name of a certain option.
-}
optionsFunctionality_getOptionTypeName :: OptionsFunctionalityClass a => a -> String -> IO String
optionsFunctionality_getOptionTypeName x = casADi__OptionsFunctionality__getOptionTypeName (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__getOptionAllowed" c_CasADi__OptionsFunctionality__getOptionAllowed
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> IO (Ptr (CppVec (Ptr GenericType')))
casADi__OptionsFunctionality__getOptionAllowed
  :: OptionsFunctionality -> String -> IO (Vector GenericType)
casADi__OptionsFunctionality__getOptionAllowed x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__getOptionAllowed x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Get the allowed values of a certain option.
-}
optionsFunctionality_getOptionAllowed :: OptionsFunctionalityClass a => a -> String -> IO (Vector GenericType)
optionsFunctionality_getOptionAllowed x = casADi__OptionsFunctionality__getOptionAllowed (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__getOptionAllowedIndex" c_CasADi__OptionsFunctionality__getOptionAllowedIndex
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> IO CInt
casADi__OptionsFunctionality__getOptionAllowedIndex
  :: OptionsFunctionality -> String -> IO Int
casADi__OptionsFunctionality__getOptionAllowedIndex x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__getOptionAllowedIndex x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Get the index into allowed options of a certain option.
-}
optionsFunctionality_getOptionAllowedIndex :: OptionsFunctionalityClass a => a -> String -> IO Int
optionsFunctionality_getOptionAllowedIndex x = casADi__OptionsFunctionality__getOptionAllowedIndex (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__setOptionByAllowedIndex" c_CasADi__OptionsFunctionality__setOptionByAllowedIndex
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> CInt -> IO ()
casADi__OptionsFunctionality__setOptionByAllowedIndex
  :: OptionsFunctionality -> String -> Int -> IO ()
casADi__OptionsFunctionality__setOptionByAllowedIndex x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__OptionsFunctionality__setOptionByAllowedIndex x0' x1' x2' >>= wrapReturn

-- classy wrapper
{-|
>Set a certain option by giving its index into the allowed values.
-}
optionsFunctionality_setOptionByAllowedIndex :: OptionsFunctionalityClass a => a -> String -> Int -> IO ()
optionsFunctionality_setOptionByAllowedIndex x = casADi__OptionsFunctionality__setOptionByAllowedIndex (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__getOptionEnumValue" c_CasADi__OptionsFunctionality__getOptionEnumValue
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> IO CInt
casADi__OptionsFunctionality__getOptionEnumValue
  :: OptionsFunctionality -> String -> IO Int
casADi__OptionsFunctionality__getOptionEnumValue x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__getOptionEnumValue x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Get the enum value corresponding to th certain option.
-}
optionsFunctionality_getOptionEnumValue :: OptionsFunctionalityClass a => a -> String -> IO Int
optionsFunctionality_getOptionEnumValue x = casADi__OptionsFunctionality__getOptionEnumValue (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__setOptionByEnumValue" c_CasADi__OptionsFunctionality__setOptionByEnumValue
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> CInt -> IO ()
casADi__OptionsFunctionality__setOptionByEnumValue
  :: OptionsFunctionality -> String -> Int -> IO ()
casADi__OptionsFunctionality__setOptionByEnumValue x0 x1 x2 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  withMarshal x2 $ \x2' ->
  c_CasADi__OptionsFunctionality__setOptionByEnumValue x0' x1' x2' >>= wrapReturn

-- classy wrapper
{-|
>Set a certain option by giving an enum value.
-}
optionsFunctionality_setOptionByEnumValue :: OptionsFunctionalityClass a => a -> String -> Int -> IO ()
optionsFunctionality_setOptionByEnumValue x = casADi__OptionsFunctionality__setOptionByEnumValue (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__getOptionDefault" c_CasADi__OptionsFunctionality__getOptionDefault
  :: Ptr OptionsFunctionality' -> Ptr StdString' -> IO (Ptr GenericType')
casADi__OptionsFunctionality__getOptionDefault
  :: OptionsFunctionality -> String -> IO GenericType
casADi__OptionsFunctionality__getOptionDefault x0 x1 =
  withMarshal x0 $ \x0' ->
  withMarshal x1 $ \x1' ->
  c_CasADi__OptionsFunctionality__getOptionDefault x0' x1' >>= wrapReturn

-- classy wrapper
{-|
>Get the default of a certain option.
-}
optionsFunctionality_getOptionDefault :: OptionsFunctionalityClass a => a -> String -> IO GenericType
optionsFunctionality_getOptionDefault x = casADi__OptionsFunctionality__getOptionDefault (castOptionsFunctionality x)


-- direct wrapper
foreign import ccall unsafe "CasADi__OptionsFunctionality__OptionsFunctionality" c_CasADi__OptionsFunctionality__OptionsFunctionality
  :: IO (Ptr OptionsFunctionality')
casADi__OptionsFunctionality__OptionsFunctionality
  :: IO OptionsFunctionality
casADi__OptionsFunctionality__OptionsFunctionality  =
  c_CasADi__OptionsFunctionality__OptionsFunctionality  >>= wrapReturn

-- classy wrapper
{-|
>Default constructor.
-}
optionsFunctionality :: IO OptionsFunctionality
optionsFunctionality = casADi__OptionsFunctionality__OptionsFunctionality

